/* Sensor Array
  By Dave Gundlach 
  Jan 2016
   
  Sensor Array are multiple sensors, each with a PIR sensor and radio tx/rx. 
  When a target trips the PIR, the sensor reports total hits.
  When another sensor picks up a report, it repeats the report.
  When all sensor recieve 2 identical reports, reporting stops.
  Each sensor needs to be in radio contact with at least one other sensor.
  Not all sensors need to be in contact with all sensors.
  Each sensor runs the same sketch with exception of variable "NODEID" (sensorNumber).
  Each sensor needs a unique number, variable "numberof" has to match the number of 
  sensor in the array.
  
  Monitor Unit 0 for recieving and displaying total hits from each sensor
  (See Mon0 Sketch)

  Use same sketch for each sensor.  Change NODEID starting with 1 for each sensor(example 1,2,3,4...).
  Change "numberof" for total sensors in array.

  Components:
  1 - Adafruit Pro Trinket - 5V 16MHZ
  1 - Adafruit RFM69HCW Transceiver Radio Breakout
  1 - PIR (motion) sensor - from Adafruit
  1 - 8 AA battery holder (12V) - from Radio Shack
  Optional Components (used for debugging in development testing
  - not required for operation)
  1 - 330 Ohm resistor for LED
  1 - LED - used for visual confermation of radio tx/rx
  1 - 10K Ohm resistor - pull down resistor for button
  1 - Momentary Contact Button - replaced PIR for testing 

  Connections:
  Pro Trinket to RFM69HCW:  |  Pro Trinket to Other:
  9 to RST                  |  Bat to Battery Pack +
  10 to CS                  |  G to PIR GND
  11 to MO                  |  5V to PIR +5V
  12 to MI                  |  8 to PIR OUT
  13 to SCK                 |
  3 to G0                   |
  G to GND                  |
  5V to VIN                 |
  
  Optional: 
  6 to 330 Ohm resistor and LED (Debug - not required)

  Libraries:
  RFM69.h  -  https://www.github.com/lowpowerlab/rfm69
  SPI.h    -  included in Arduino IDE
  
  HISTORY 
  001 - Basic Code
  002 - Took out sensor number from the report. report is now: 
        Reporting Sensor#,Sensor 1 hits,Sensor 2 hits,...
  003 - Added Reset
  004 - added test hits
  005 - For PIR sensor, no test hits, no button
  006 - For RFM69HCW Radio (feather)
  007 - For RFM69HCW Radio (pro trinket)
  008 - Ignore PIR during transmit to prevent false triggers.  Reset couter at 99.
  009 - Clean Up Code
 */

#include <RFM69.h>    //get it here: https://www.github.com/lowpowerlab/rfm69
#include <SPI.h>

#define NETWORKID     101                 // The same on all nodes that talk to each other
#define NODEID        1                   // The unique identifier of this node **change value for each sensor**
#define RECEIVER      255                 // The recipient of packets (broadcast)
#define FREQUENCY     RF69_433MHZ         // Use the frequency of the radio unit you are using
#define ENCRYPTKEY    "whereIsTHEfoxHat"  // Use same key for all sendor in the array
#define IS_RFM69HCW   true 
#define RFM69_CS      10
#define RFM69_IRQ     3
#define RFM69_IRQN    1                   // Pin 3 is IRQ 1!
#define RFM69_RST     9

int sensorNumber = NODEID;                // Sensor Number
int numberof = 4;                         // Number of sensors in array **change for total number of sensors**
int sensorPin = 8;                        // PIR input pin
int ledPin = 6;                           // LED input pin (Optional - not required)
int sensorValue = 0;                      // PIR sensor value
int sensorTotal = 0;                      // PIR total hits
int sv[] = {0, 222, 222, 222, 222, 222, 222, 222, 222};     // sensor hit values
int svLast[] = {0, 222, 222, 222, 222, 222, 222, 222, 222}; // sensor last hit values
unsigned long comTimer=0;                 // Communication Timer
//unsigned long testTimer;                // FOR TESTING ONLY
//byte testFlag = 1;                      // FOR TESTING ONLY   
byte rFlag = 0;                           // Report Flag
byte pirFlag = 1;                         // PIR Flag - only one hit for detect
                                          // and ignore PIR during transmit
                                          
RFM69 radio = RFM69(RFM69_CS, RFM69_IRQ, IS_RFM69HCW, RFM69_IRQN);

void setup() {
  Serial.begin(115200);                     // Start hardwire serial for debug
  pinMode(RFM69_RST, OUTPUT);               // Hard Reset the RFM module
  digitalWrite(RFM69_RST, HIGH);
  delay(100);
  digitalWrite(RFM69_RST, LOW);
  delay(100);
  radio.initialize(FREQUENCY,NODEID,NETWORKID);   // Initialize radio
  if (IS_RFM69HCW) {
    radio.setHighPower();                         // Only for RFM69HCW & HW!
  }
  radio.setPowerLevel(31);                  // power output ranges from 0 (5dBm) to 31 (20dBm)
  radio.encrypt(ENCRYPTKEY);
  pinMode(sensorPin, INPUT);
  pinMode(ledPin, OUTPUT);
}

void loop() {

  checkRadio();          // Check Radio function
  if (rFlag){            // Report if Flag is TRUE
    relayReport();       // Relay Report function
  }
  checkSensor();         // Check Sensor function
}
//******************* Check Radio Function ***********************************

void checkRadio(){
  uint8_t buf[RF69_MAX_DATA_LEN];              // Reciever Buffer
  uint8_t buflen = radio.DATALEN;              // Buffer Length
  int value[10];                               // Integer Value
  int count = numberof+1;                      // Count of data
  char valueChar[4];                           // Char Value

  if (millis() > (comTimer + numberof*1000 + 1000)) {   // Check Tranmit timer
    comTimer = millis();                                // Reset timer
  }
  if (radio.receiveDone()){
    for(int i=0; i<buflen; i++){                        // Fill recieve buffer
      buf[i]=radio.DATA[i];
      Serial.println(buf[i]);
    }
    Serial.println((char*)radio.DATA);              // Debug
    for(int j=0; j<3; j++){                         // data has 3 digits
      valueChar[j]=char(buf[j]);                    // make ascii char values
    }
    value[0] = atoi(valueChar)-100;                 // convert char to int
    if (value[0]<100){
      comTimer = millis()-(value[0]*1000);          // start com timer
      digitalWrite(ledPin, HIGH);                   // LED on
      Serial.print("Values Recieved: ");            // Debug
      for (int i = 0; i < buflen; i++) {
        Serial.print(char(buf[i]));                 // Debug
        if (i==2||i==5||i==8||i==11||i==14||i==17) Serial.print(" ");
      }
      Serial.println("");
      for (int i=0; i<count; i++){                  // for message data count
        for(int j=0; j<3; j++){                     // data has 3 digits
          valueChar[j]=char(buf[j+(i*3)]);          // make ascii char values
        }
        value[i] = atoi(valueChar)-100;             // convert char to int
      }
      for (int i=1; i<=numberof; i++){              // for sensor values in message
        if (sv[i]==222){                            // only overwrite unknown values
          sv[i] = value[i];                         // put sensor values in aray
        }
        if (sv[i]==223){                            // Reset Command
          sv[i] = 223;                              // Relay Reset Command
          sensorTotal=0;                            // Reset Sensor Total
          sv[sensorNumber]=0;
        }
      }
      if (sv[sensorNumber]==222){                   // if other sensor did not recieve
        rFlag = 1;                                  // transmit again
        sv[sensorNumber] = sensorTotal;             // update this sensors value
      }
      else {
        rFlag = 0;                                  // set report flag to false
        for (int i = 1; i <= numberof; i++){        // for each senor
          if (sv[i] != svLast[i]){                  // if any value changed from last report
            rFlag = 1;                              // set report flag to true
          }
        }
      }
      
      for (int i = 1; i <= numberof; i++) {         // put values into svLast[]
        svLast[i] = sv[i];
      }
      digitalWrite(ledPin, LOW);                    // LED off
      for (int i=0; i<count; i++){
        valueChar[1]=0;                             // clear array 
      }
    }
    if (!rFlag){
      for (int i=1; i<=numberof; i++){
        sv[i]=222;                                  // reset sensor values to unknown
      } 
    } 
  }
}



//******************* Relay Report Function ***********************************
// This function relays new report to radio
void relayReport(){
  char numChar[4];                          // Char number
  int count = numberof+1;                   // Count of data numbers
  int sn=0;                                 // Int sensor number
  int num[50];                              // Int combined number array
  char msg[50];                             // Char combined number for message 
  
  if (millis() > (comTimer + sensorNumber * 1000)
  && millis() < (comTimer + sensorNumber * 1000 + 1000)) {
    pirFlag=0;                              // ignore PIR when transmitting
    digitalWrite(ledPin, HIGH);             // LED on
    Serial.print(sensorNumber);             // Debug report
    Serial.print(" Send Report at: ");
    Serial.println(millis() - comTimer);
    sv[sensorNumber] = sensorTotal;         // update this sensors value
    num[0]=sensorNumber;                    // add sensor number to int array       
    for (int i=1; i<count; i++) {           // build int array 
      num[i] = sv[i];                       // add sensor value
    }
    for (int i=0; i<count; i++){            // build char array
      itoa(num[i]+100, numChar, 10);        // add 100 to value and convert int to char
        for (int j=0; j<3; j++){            // for each 3 digit char
        msg[j+(i*3)] = numChar[j];          // put char into message
      }
    }
    msg[count*3]='\0';                      // end msg with null
    Serial.print(".");
    radio.send(RECEIVER,msg, strlen(msg));  //target node Id, message as string or byte array, message length
    Serial.print(".");
    radio.receiveDone();                    //put radio in RX mode
    Serial.flush();                         //make sure all serial data is clocked out before sleeping the MCU
    Serial.println(".");
    rFlag = 0;                              // Relay Flag false
    digitalWrite(ledPin, LOW);              // LED off
       
  }
}

//******************* Check Sensor Function ***********************************
// Check sensor for a new hit and transmits data
void checkSensor(){
  if (digitalRead(sensorPin) && pirFlag) {      // if sensor has a hit
    pirFlag=0;
    digitalWrite(ledPin, HIGH);                 // LED on
    Serial.print(sensorNumber);                 // debug
    Serial.print(" Sends Hit ==> ");
    sensorTotal = sensorTotal + 1;              // increment total hits
    if (sensorTotal > 99) sensorTotal = 0;      // reset hits if over 99
    
    delay(1000);
    rFlag=1;
    digitalWrite(ledPin, LOW);                  // LED off
    comTimer = millis()- sensorNumber * 1000;  
    relayReport();                              // send report
  }
  if (!digitalRead(sensorPin)) pirFlag=1; 
}

